# code adapted from http://www4.stat.ncsu.edu/~reich/st590/code/CV

rm(list=ls())
library(foreign)
library(BRugs)
library(arm)
library(R2WinBUGS)
library(rjags)
library(runjags)

dat<-read.dta(file="gay_attitudes.dta")
dat$state<-ifelse(dat$state=="", NA, dat$state)
dat<-subset(dat, year==2008)
dat<-na.omit(dat)
dat$churchattend<-as.numeric(dat$church)
dat$partyid<-as.numeric(dat$partyid)

table(dat$year)

N<-dim(dat)[1]

dat$state.ind<-c()
for(i in 1:length(dat$state)){
  dat$state.ind[i]<-which(unique(dat$state)==dat$state[i])
}

K<-length(unique(dat$state.ind))

dat$state.name<-c()
for(i in 1:K){
  dat$state.name[i]<-unique(dat$state[which(dat$state.ind==i)])
}


# group mean centering
age.means <- aggregate(dat$age, by=list(dat$state.ind), FUN=mean)[,2]
dat$age.center <- dat$age - age.means[dat$state.ind]
partyid.means <- aggregate(dat$partyid, by=list(dat$state.ind), FUN=mean)[,2]
dat$partyid.center <- dat$partyid - partyid.means[dat$state.ind]
churchattend.means <- aggregate(dat$churchattend, by=list(dat$state.ind), FUN=mean)[,2]
dat$churchattend.center <- dat$churchattend - churchattend.means[dat$state.ind]

dat$female <- as.numeric(dat$gender)-2
dat$black <- as.numeric(as.numeric(dat$race)==2)

set.seed(654321)
test.ind <- sample(1:dim(dat)[1], 100, F)
train.ind <- which( !(1:dim(dat)[1] %in% test.ind) == T)
dat.train <- dat[train.ind, ]
dat.test <- dat[test.ind, ]

dat <- dat.train
N<-dim(dat)[1]


set.seed(123456)
jags.dat<- list("N"=N, "K"=K, "gaytherm"=dat$gaytherm, "age"=dat$age.center, "state.ind"=dat$state.ind, "churchattend"=dat$churchattend.center, "partyid"=dat$partyid.center, "female" = dat$female, "black" = dat$black,
                "age.test"=dat.test$age.center, "state.ind.test"=dat.test$state.ind, "churchattend.test"=dat.test$churchattend.center, "partyid.test"=dat.test$partyid.center, "female.test" = dat.test$female, "black.test" = dat.test$black)

jags.inits <- function() {list (b.age = 0, b.church = 0, b.party=0, b.female=0, b.black=0, tau.y=1)}
parameters<- c("gaytherm.test")
reg.jags<-jags.model(file="gay_attitudes_cv.bug", data=jags.dat, inits=jags.inits, n.chains=10, n.adapt=1000)
update(reg.jags, n.iter=2000) # burn in
regression.sim<-coda.samples(reg.jags, variable.names=parameters, n.iter=2000)

summary(regression.sim)

cv.samples <- regression.sim[[1]]

test.mat <- matrix( dat = dat.test$gaytherm, byrow=T, nrow=2000, ncol=100)

# mean square error
mean( (cv.samples -  test.mat)^2 )

# bias
mean( (cv.samples -  test.mat) )

# absolute/magnitude bias
mean( abs(cv.samples -  test.mat) )











set.seed(123456)
jags.2.dat<- list("N"=N, "K"=K, "gaytherm"=dat$gaytherm, "age"=dat$age.center, "state.ind"=dat$state.ind, "churchattend"=dat$churchattend.center, "partyid"=dat$partyid.center, "female" = dat$female,
                "age.test"=dat.test$age.center, "state.ind.test"=dat.test$state.ind, "churchattend.test"=dat.test$churchattend.center, "partyid.test"=dat.test$partyid.center, "female.test" = dat.test$female)

jags.2.inits <- function() {list (b.age = 0, b.church = 0, b.party=0, b.female=0, tau.y=1)}
parameters<- c("gaytherm.test")
reg.2.jags<-jags.model(file="gay_attitudes_cv2.bug", data=jags.2.dat, inits=jags.2.inits, n.chains=10, n.adapt=1000)
update(reg.2.jags, n.iter=2000) # burn in
regression.2.sim<-coda.samples(reg.2.jags, variable.names=parameters, n.iter=2000)

cv.samples.2 <- regression.2.sim[[1]]

# mean square error
mean( (cv.samples.2 - test.mat)^2 )
mean( (cv.samples - test.mat)^2 )


# bias
mean( (cv.samples.2 - test.mat) )
mean( (cv.samples - test.mat) )


# absolute/magnitude bias
mean( abs(cv.samples.2 -  test.mat) )
mean( abs(cv.samples -  test.mat) )



dic.1 <- dic.samples(model=reg.jags, n.iter=1000, type="pD")
dic.2 <- dic.samples(model=reg.2.jags, n.iter=1000, type="pD")

diffdic(dic.1, dic.2)